# $Id: Conf.pm 85 2005-06-29 00:18:21Z gwolf $

package IPmanage::Conf;

=head1 NAME

IPmanage::Conf - Handler for IPmanage's configuration

=head1 SYNOPSIS

  $conf = IPmanage::Conf->new($db);

  @keys = $conf->get_conf_keys;

  @keys = get_basic_keys;
  $ok = $conf->is_basic($key);

  $val = $conf->get_val($key);
  $ok = $conf->set_val($key, $val);

  $ok = $conf->create_key($key, $val);
  $ok = $conf->del_key($key);

  $ok = $conf->populate_basic_keys;

=head1 DESCRIPTION

  $conf = IPmanage::Conf->new($db);

Creates a new IPmanage::Conf object. Receives an open DBI connection as its
only parameter.

  @keys = $conf->get_conf_keys;

Gets the list of currently defined configuration keys in the database.

  @keys = get_basic_keys;

Gets the list of the basic keys (i.e., those required for IPmanage's correct
operation, which should be always present)

  $ok = $conf->is_basic($key);

Queries if a specific key is within the basic keys

  $val = $conf->get_val($key);
  $ok = $conf->set_val($key, $val);

Retrieves or modifies the value of the specified configuration key.

  $ok = $conf->create_key($key, $val);

Creates the specified configuration key with the specified value.

  $ok = $conf->del_key($key);

Removes the specified configuration key.

  $ok = $conf->populate_basic_keys;

Verifies all the required configuration keys are present in the system, and
creates them (with their default values if applicable) if needed. The 
configuration keys in question are:

    dhcp_default_lease   (default: 3600)
    dhcp_dns_server 
    dhcp_domain_name 
    dhcp_max_lease       (default: 28800)
    dhcp_netbios_server 
    dhcp_site_name 
    server_ips           (default: 127.0.0.1)
    server_notif_addr    (default: 127.0.0.1:32000)
    server_notif_pass

If the configuration keys already exist, they will be left alone, even if they
have an empty value and there is a default.

=head1 SEE ALSO

L<IPmanage>

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms 
as Perl.

=cut

use warnings;
use strict;
use Carp;

sub new {
    my ($self, $class, $db);
    $class = shift;
    $db = shift;
    $self = { db => $db };

    bless $self, $class;

    # Just as a sanity check, this is a good place to populate the basic keys
    $self->populate_basic_keys;

    return $self;
}

sub get_conf_keys {
    my ($self, $sth, @keys);
    $self = shift;

    $sth = $self->{db}->prepare('SELECT name FROM config');
    $sth->execute;
    @keys = map { $_->[0] } @{$sth->fetchall_arrayref};

    return @keys;
}

sub get_basic_keys {
    my %keys = _basic_keys_hash();
    return sort keys %keys;
}

sub is_basic {
    my ($self, $key, %basic);
    $self = shift;
    $key = shift;

    %basic = _basic_keys_hash();

    return 1 if grep {$key eq $_} keys %basic;
    return 0;
}

sub get_val {
    my ($self, $key, $sth);
    $self = shift;
    $key = shift;

    $sth = $self->{db}->prepare('SELECT value FROM config WHERE name = ?');
    $sth->execute($key);
    return $sth->fetchrow_array;
}

sub set_val {
    my ($self, $key, $val, $sth, $res);
    $self = shift;
    $key = shift;
    $val = shift;

    # Some keys' format can be validated - Do it!
    if (grep {$key eq $_} qw(dhcp_default_lease dhcp_max_lease)) {
	# Require a number, 3 to 6 digits long
	if ($val !~ /^\d{3,6}$/) {
	    carp "Value ($val) should be a number for $key";
	    return 0;
	}
    } elsif ($key eq 'server_ips') {
	# Require something looking as an IP address or set of IP addresses
	# separated by commas (and possibly spaces)
	if ($val !~ /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s*,?\s*)+$/) {
	    carp "Value ($val) should be an IP list for $key";
	    return 0;
	}
    } elsif ($key eq 'server_notif_addr') {
	# Require something looking as an IP address followed by its port, or
	# a list of such things
	if ($val !~ /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\:\d{1,5}\s*,?\s*)+$/) {
	    carp "Value ($val) should be an IP:port list for $key";
	    return 0;
	}
    }

    $sth = $self->{db}->prepare('UPDATE config SET value = ? WHERE name = ?');
    $res = $sth->execute($val, $key);
    return ($res == '0E0') ? 0 : 1;
}

sub create_key {
    my ($self, $key, $val, $sth);
    $self = shift;
    $key = shift;
    $val = shift;

    $sth = $self->{db}->prepare('INSERT INTO config (name, value)
        VALUES (?, ?)');
    return $sth->execute($key, $val);
}

sub del_key {
    my ($self, $key, $sth, $res);
    $self = shift;
    $key = shift;

    # Don't remove basic keys
    return 0 if grep {$key eq $_} $self->get_basic_keys;

    $sth = $self->{db}->prepare('DELETE FROM config WHERE name = ?');
    $res = $sth->execute($key);
    return ($res == '0E0') ? 0 : 1;
}

sub populate_basic_keys {
    my ($self, %defaults, @existing);
    $self = shift;

    %defaults = _basic_keys_hash();
    @existing = $self->get_conf_keys;

    for my $key (keys %defaults) {
	next if grep {$key eq $_} @existing;

	$self->create_key($key, $defaults{$key});
    }
}

######################################################################
# Private methods, not for human consumption

# Returns the hash of basic keys, with their default values.
sub _basic_keys_hash {
    return (dhcp_default_lease => 3600,
	    dhcp_dns_server => '',
	    dhcp_domain_name => '',
	    dhcp_max_lease => 28800,
	    dhcp_netbios_node_type => 8,
	    dhcp_netbios_server => '',
	    dhcp_site_name => '',
	    server_ips => '127.0.0.1',
	    server_notif_addr => '127.0.0.1:32000',
	    server_notif_pass => '');
}

1;

# $Log$
# Revision 1.6  2005/06/29 00:18:21  gwolf
# Added support for hostnames
#
# Revision 1.5  2005/05/10 05:05:25  gwolf
# Validate some fields
#
# Revision 1.4  2005/05/05 18:28:16  gwolf
# - Completed the documentation
# - Added the get_basic_keys and is_basic methods
#
# Revision 1.3  2005/04/14 00:03:41  gwolf
# Continuing with the translation, it _seems_ everything is working as it should
#
# Revision 1.2  2005/04/05 01:18:01  gwolf
# Translated to English.
# Have not tested functionality yet, but at least, it compiles. It deserves
# being shipped!
#
