# $Id: Networks.pm 75 2005-05-17 01:19:58Z gwolf $

package IPmanage::Networks;

=head1 NAME

IPmanage::Networks - 

=head1 SYNOPSIS

  $nets = IPmanage::Networks->new($db);

  $net = $nets->get_net( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->get_name( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->get_gateway( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->is_dynamic( ( id => $id | net => $net | ip => $ip ) );

  $net = $nets->get_data( ( id => $id | net => $net | ip => $ip ) );

  $net = $nets->ip_belongs_to($ip);

  $status = $nets->has_static_info($id);

  @nets = $nets->list;

  $err = $nets->error;

  $ok = $nets->set_name($id, $name);
  $ok = $nets->set_gateway($id, $gateway);
  $ok = $nets->set_dynamic($id);
  $ok = $nets->unset_dynamic($id);

  $ok = $nets->create(net => $net_addr, name => $net_name, 
                      gateway => $net_gw, dynamic => $dynamic);

  $ok = $nets->delete(net => $net_addr);
  $ok = $nets->delete(id => $id);

  $ok = $self->limits;
  $ok = $self->no_limits;

=head1 DESCRIPTION

=head2 Constructor

  $net = IPmanage::Networks->new($db)

$db should be an open connection to a database (using DBI).

=head2 Retreiving/modifying information

  $net = $nets->get_net( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->get_name( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->get_gateway( ( id => $id | net => $net | ip => $ip ) );
  $net = $nets->is_dynamic( ( id => $id | net => $net | ip => $ip ) );

Returns, respectively, the network address, network name (description), gateway
address or whether the network is dynamically assigned. The network in 
question is requested via its database ID, the full network/masklen 
specification or an IP address belonging to it

  $net = $nets->get_data( ( id => $id | net => $net | ip => $ip ) );

Returns a hash reference with the information for the specified network 
(using the same query format as in the above block) using the following 
return format:

  {id=>$id, name=>$name, net=>$net, gateway=>$gateway, dynamic=>$dynamic,
   first_ip=>$ip1, last_ip=>$ip2}

  $net = $nets->ip_belongs_to($ip);

Queries which network (that is, netaddr/masklen) does the specified IP address
belong to. Returns undef if no network containing the specified address can be
found in the database.

  $status = $nets->has_static_info($id)

If the network whose identifier we received has any IPs with static data, 
returns a true value. This value can be seen as boolean (0=false, anything 
else=true), or it can be seen as a descriptive value: If it is a positive
integer, it means some IPs (the returned number) have MAC address defined
for them and are not blocked (this means, they are configured for static 
operation). If it is a negative number, no IPs are fully configured to be 
used right now, but some IPs have data in them (i.e., location, responsible,
even a MAC address but status is blocked). Again, the number returned is the
amount of relevant addresses.

Of course, the most relevant information wins: If there is only one unblocked
MAC address, 1 will be returned regardless of how many unconfigured addresses 
with data are there.

Note that, as we are using 0 as a meaningful, important value, in case this
method finds an error, we will return a non-standard and ugly '-err-' string.
Yes, it is ugly, but you have been warned. Any suggestions are welcome :)

  @nets = $nets->list;

Gets a complete listing of the networks we have in our system. Each of the 
returned elements has the same data and format as that received via the
C<get_data> method.

    $err = $nets->error;

Gets the last error message encountered by the module.

  $ok = $nets->set_name($id, $name);
  $ok = $nets->set_gateway($id, $gateway);
  $ok = $nets->set_dynamic($id);
  $ok = $nets->unset_dynamic($id);

Sets each of the specified attributes to the specified value (C<set_dynamic> 
and C<unset_dynamic> toggle the booleam C<dynamic> field). Please note that
the database will not accept an unreachable gateway, so C<set_gateway> must
have a value that is inside the network in question.

=head2 Creating and removing networks

  $ok = $nets->create(net => $net_addr, name => $net_name, 
                      gateway => $net_gw, [dynamic => $dynamic]);

Creates a network with the specified data, and populates the database with the
IP addresses belonging to it.

By default, this module will allow you to create network blocks with a netmask
of 24 bits or more (that is, 256 IP addresses or less). If you want to create a
larger network, you have to specify L<no_limits>.

The only optional parameter is C<dynamic> (Boolean value. True: This module
will hand out dynamic IPs via DHCP. False: Only static IP addresses will be
accepted), it defaults to false.

  $ok = $nets->delete(net => $net_addr);
  $ok = $nets->delete(id => $id);

Removes the specified network (and all of its IPs) from the system. 

Be careful: This operation cannot be undone!

=head2 Modifying module behavior

  $ok = $nets->limits;
  $ok = $nets->no_limits;

Sets/unsets sanity checks on the limits for this module's operation. Right now,
the only action affected by this is L<create>'s ability to create networks with
over 256 IPs.

=head1 EXTRA

=head2 Masklen?

Throughout this documentation we refer to the I<masklen> instead of the much 
more typical term I<netmask>. Why?

A netmask should be I<visually> enough to know approximately where is the
network/host boundary in a particular address - So for example,
172.16.8.0/255.255.255.0 cleraly shows that the netmask covers the three most
significative octets, while the last one is the host portion. This same
network address can be specified using the masklen notation as 172.16.8.0/24.
Not only the masklen notation is much shorter, it is also the notation chosen
to be used in Postgres. So you'd better like it, because I will not change it.

And, yes, I recognize this last paragraph has a bit of pedantry, of mostly 
needless precision... Well, yes, I am guilty ;-)

=head1 SEE ALSO

L<IPmanage>

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms 
as Perl.

=cut

use warnings;
use strict;
use Carp;
use IPmanage::IP;
use Net::IP;

######################################################################
# Constructor

sub new {
    my ($self, $class, $db, $sth);
    $class = shift;
    $db = shift;

    $self = { db => $db, no_limits => 0 };
    bless $self, $class;

    $self->refresh;

    return $self;
}

######################################################################
# Retreiving/modifying information

sub get_net {
    my ($self, $identifier, $which, $data);
    $self = shift;
    $identifier = shift;
    $which = shift;

    $data = $self->get_data($identifier, $which) or return undef;

    return $data->{net};
}

sub get_name {
    my ($self, $identifier, $which, $data);
    $self = shift;
    $identifier = shift;
    $which = shift;

    $data = $self->get_data($identifier, $which) or return undef;

    return $data->{name};
}

sub get_gateway {
    my ($self, $identifier, $which, $data);
    $self = shift;
    $identifier = shift;
    $which = shift;

    $data = $self->get_data($identifier, $which) or return undef;

    return $data->{gateway};
}

sub is_dynamic {
    my ($self, $identifier, $which, $data);
    $self = shift;
    $identifier = shift;
    $which = shift;

    $data = $self->get_data($identifier, $which) or return undef;

    return $data->{dynamic};
}

sub get_data {
    my ($self, $identifier, $which, $res);
    $self = shift;
    $identifier = shift || '';
    $which = shift || '';

    # If we are being queried for an IP, find out the network to which it
    # belongs and work with that.
    if ($identifier eq 'ip') {
	my $net = $self->ip_belongs_to($which);
	unless (defined $net) {
	    $self->_set_err('Could not find the network under which IP ' .
			    $which . 'is registered');
	    carp $self->error;
	    return undef;
	}

	return $self->get_data(net => $net);
    }

    if ($identifier ne 'net' and $identifier ne 'id') {
	$self->_set_err('Invocation error');
	carp $self->error;
	return undef;
    }

    # There can be only a single result, don't worry about more than that
    for my $net (0..$#{$self->{nets}}) {
	if ($self->{nets}[$net]{$identifier} eq $which) {
	    $res = $self->{nets}[$net];
	    last;
	}
    }

    return $res;
}

sub ip_belongs_to {
    my ($self, $ip, $sth, $net);
    $self = shift;
    $ip = shift;

    unless ($sth = $self->{db}->prepare('SELECT netaddr FROM network WHERE
            netaddr >> ?') and $sth->execute($ip)) {
	$self->_set_err("Error looking up network to which $ip belongs");
	carp $self->error;
	return undef;
    }
    ($net) = $sth->fetchrow_array;

    return $net;
}

sub has_static_info {
    my ($self, $id, $net, $sth, $res);
    $self = shift;
    $id = shift;
    $net = $self->get_net(id => $id);

    unless ($sth = $self->{db}->prepare('SELECT count(ip) FROM ip WHERE 
            ip << ? AND mac IS NOT NULL AND NOT blocked') and
	    $sth->execute($net)) {
	$self->_set_err('Error retreiving fully configured static IPs within' .
			"network $id");
	carp $self->error;
	# check documentation above for an explanation of this error value
	return '-err-'; 
    }
    ($res) = $sth->fetchrow_array;

    return $res if $res;

    # Ok, no IPs fully configured. Any IPs half-configured?
    unless ($sth = $self->{db}->prepare("SELECT count(ip) FROM ip WHERE 
            ip << ? AND (mac IS NOT NULL OR location IS NOT NULL OR responsible
            IS NOT NULL OR details IS NOT NULL OR inventory IS NOT NULL)") and
	    $sth->execute($net)) {
	$self->_set_err('Error retreiving half configured static IPs within' .
			"network $id");
	carp $self->error;
	# check documentation above for an explanation of this error value
	return '-err-'; 
    }
    ($res) = $sth->fetchrow_array;

    return 0-$res;    
}

sub list {
    my ($self);
    $self = shift;

    return @{$self->{nets}};
}

sub error {
    my ($self);
    $self = shift;

    return $self->{error};
}

sub set_name {
    my ($self, $id, $name);
    $self = shift;
    $id = shift;
    $name = shift;
    return $self->_set_attr('name', $id, $name);
}

sub set_gateway {
    my ($self, $id, $gw);
    $self = shift;
    $id = shift;
    $gw = shift;
    return $self->_set_attr('gateway', $id, $gw);
}

sub set_dynamic {
    my ($self, $id);
    $self = shift;
    $id = shift;
    return $self->_set_attr('dynamic', $id, 1);
}

sub unset_dynamic {
    my ($self, $id);
    $self = shift;
    $id = shift;
    return $self->_set_attr('dynamic', $id, 0);
}

######################################################################
# Creating and removing networks

sub create {
    my ($self, %netdata, $netip);
    $self = shift;
    %netdata = @_;

    unless (defined $netdata{name} and defined $netdata{net}) {
	$self->_set_err('Wrong parameters');
	carp $self->error;
	return undef;
    }
    if (grep {$_ !~ /^(name|gateway|net|dynamic)$/} keys %netdata) {
	$self->_set_err('Wrong parameters');
	carp $self->error;
	return undef;
    }

    # Sanitize whatever we get on dynamic to a nice boolean 0 or 1.
    $netdata{dynamic} = 0 unless defined $netdata{dynamic};
    $netdata{dynamic} = 1 if $netdata{dynamic};

    # We accept undef for gateway, but it gets sometimes through as '' -
    # clean it as well
    undef $netdata{gateway} if $netdata{gateway} eq '';

    # Yes, this regex is way too simple to ensure this is a valid IP, and we 
    # anyway creating a Net::IP object - Why? Because we want here to check
    # that we have something that looks like a CIDR network - a.b.c.d/e, 
    # with 0<=e<=32
    unless ($netdata{net} =~ m!^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/(\d+)$! and
	    $1 >= 0 and $1 <= 32 and $netip = Net::IP->new($netdata{net})) {
	$self->_set_err("Specified network has an invalid format");
	carp $self->error;
	return undef;
    }
    if ($1 < 24 and ! $self->{no_limits}) {
	$self->_set_err("Netmasks under 24 bits not supported unless 'no_limits' is set");
	carp $self->error;
	return undef;
    }

    $self->{db}{RaiseError} = 1;
    eval {
	my ($sth, $ipaddr);
	$self->{db}->begin_work;

	# First insert the network itself...
	$sth = $self->{db}->prepare('INSERT INTO network (name, netaddr, 
            gateway, dynamic) VALUES (?, ?, ?, ?)');
	$sth->execute($netdata{name}, $netdata{net},
		      $netdata{gateway}, $netdata{dynamic});

	# Now the individual IPs. We will not add the network boundaries 
	# (first and last addresses), as they are reserved.
	while (++$netip) {
	    last if $netip->ip eq $netip->last_ip;
	    my $ip = IPmanage::IP->create($self->{db}, $netip->ip);
	}

	$self->{db}->commit;
	$self->{db}->{RaiseError} = 0;
    };
    if ($@) {
	$self->{db}->rollback;
	$self->_set_err("Error registering requested network ($netdata{net}): $@");
	carp $self->error;
	return undef;
    }

    $self->refresh;

    return 1;
}

sub delete {
    my ($self, $identifier, $which, $sql, $sth);
    $self = shift;
    $identifier = shift;
    $which = shift;

    if ($identifier ne 'net' and $identifier ne 'id') {
	$self->_set_err('Invocation error');
	carp $self->error;
	return undef;
    }
    # Ugly, yes... Sorry :)
    $identifier = 'netaddr' if $identifier eq 'net';

    # Don't worry about deleting each of the IPs in the network - it will be
    # handled by a trigger in the DB
    $sth = $self->{db}->prepare("DELETE FROM network WHERE $identifier = ?");
    unless ($sth->execute($which)) {
	$self->_set_err("Could not delete the requested network ($which)");
	carp $self->error;
	return undef;
    }

    $self->refresh;

    return 1;
}

######################################################################
# Modifying module behavior

sub limits {
    my $self = shift;
    $self->{no_limits} = 0;
    return 1;
}

sub no_limits {
    my $self = shift;
    $self->{no_limits} = 1;
    return 1;
}

######################################################################
# Private methods

# Reloads the data defined in the object to be in sync with the DB
sub refresh {
    my ($self, $sth, @nets);
    $self = shift;

    unless ($sth = $self->{db}->prepare('SELECT id, name, netaddr, gateway, 
            dynamic FROM network ORDER BY netaddr') and $sth->execute) {
	$self->_set_err("Could not retreive network information");
	carp $self->error;
	return undef;
    }

    while (my @row = $sth->fetchrow_array) {
	# Not only we get the data itself, but some stuff often used which 
	# can be calculated from it. Get Net::IP!
	my $netip = Net::IP->new($row[2]);

	push @nets, {id => $row[0],
		     name => $row[1], 
		     net => $row[2],
		     gateway => $row[3],
		     dynamic => $row[4],
		     netmask => $netip->mask,
		     netaddr => $netip->ip,
		     last_ip => $netip->last_ip
		 };
    }

    $self->{nets} = \@nets;

    return 1;
}

# Sets the specified attribute in the DB to the specified value
sub _set_attr {
    my ($self, $field, $id, $value, $sth);
    $self = shift;
    $field = shift;
    $id = shift;
    $value = shift;

    unless ($field =~ /^(name|gateway|dynamic)$/) {
	$self->_set_err('Invocation error');
	carp $self->error;
	return undef;
    }

    # We often receive empty strings when we should get nulls - fix it.
    undef $value if $value eq '';

    unless ($sth = $self->{db}->prepare("UPDATE network SET $field = ? WHERE 
            id = ?") and $sth->execute($value, $id)) {
	$self->_set_err("Could not set $field to $value");
	carp $self->error;
	return undef;
    }

    $self->refresh;

    return 1;
}

# Sets the specified error message
sub _set_err {
    my ($self, $error);
    $self = shift;
    $error = shift;
    $self->{error} = $error;
    return 1;
}

1;

# $Log$
# Revision 1.4  2005/05/17 01:19:58  gwolf
# Allow for gateways not to be located on the network boundary on dynamic networks
#
# Revision 1.3  2005/05/02 19:11:12  gwolf
# Basically a rewrite, allowing for a more flexible use
#
# Revision 1.2  2005/04/19 01:19:44  gwolf
# - More fine-tuning in network administration
# - Some more sanity checking in plpgsql
# - More network-related method (what was it? hmmm... Yup, checking if a
#   network has any static configurations set)
# - Firsts steps towards gen_dhcpd_conf
#
# Revision 1.1  2005/04/16 03:31:22  gwolf
# Functionality for managing networks. This is basically done, yay!
#
