#!/usr/bin/perl
use warnings;
use strict;
use Config::Simple;
use Digest::MD5 qw(md5_hex);
use Getopt::Long;
use IO::File;
use IO::Socket;
use LWP::UserAgent;
use Net::SMTP;
use POSIX;
use Text::Diff;

$SIG{CHLD} = 'IGNORE';
my ($conffile, $conf, $sock, $new_conf, $old_conf);

$conffile = get_conffile();
$conf = Config::Simple->import_from($conffile);

# Make sure we are root - Exit otherwise.
die "$0 must be run as root" unless $< == 0;

_debug('Starting server');
# Start the server socket
$sock = IO::Socket::INET->new(Listen => 5,
			      LocalPort => $conf->param('listen_port'),
			      Proto => 'tcp',
			      Type => SOCK_STREAM)
    or die 'Cannot open socket on port ', $conf->param('listen_port'),": $!";

# Give up this process control: Fork and exit
exit 0 if fork();

_debug('Waiting for incomming connections');
# Handle incoming requests
while (my $conn = $sock->accept) {
    my ($ip, $port);
    $ip = inet_ntoa($conn->peeraddr);
    $port = $conn->peerport;
    _debug("Got a new connection from $ip:port");

    # Fork, if this is the parent go to wait for a new connection
    next if fork;

    # Make sure we are talking to the right client
    authenticate($conn) or die "Authentication failed from $ip:$port\n";
    _debug('Authenticated successfully');

    # We don't need no stinky connections anymore
    $conn->close;

    _debug('Getting new daemon configuration');
    # Get the new daemon configuration
    $new_conf = get_new_conf($conf->param('conf_url'));
    $old_conf = get_old_conf($conf->param('conffile'));

    # Is it different from the current configuration?
    if (my $conf_diff = diff(\$new_conf, \$old_conf)) {
	_debug('Daemon configuration is different from current one');
        # Verify file is not too small (set an arbitrary minimum of 250 bytes
        # for it to be valid)
	if (length($new_conf) < 250) {
	    die "Config length too small (" . length($new_conf) .
		" bytes)- Is this a valid file?\n";
	}
    
	_debug('Notifying administrator');
        # First, notify the administrator of any changes
	send_admin_mail($conf->param('send_mail_from'), 
			$conf->param('send_mail_to'), 
			$conf->param('smtp_server'), $conf_diff);

	_debug('Putting file in its place');
        # Put the file in place
	write_new_conf($conf->param('conffile'), $new_conf);

	_debug('Restarting daemon');
        # Restart daemon
	system($conf->param('restart_cmd'));
    } else {
	# Same file - nothing else to do.
	_debug('Configuration file already up to date');
	exit 0;
    }
}

sub authenticate {
    # Send a random challenge to the client. Read its response. If the response
    # is not valid (this means, a MD5 of the challenge and the password), kill 
    # the connection.
    my ($conn, $challenge, $client_str);
    $conn = shift;

    $challenge = _gen_challenge();
    _debug("Sending challenge $challenge");
    $conn->print("$challenge\n");
    $client_str = $conn->getline;; 
    # Do this instead of a chomp, as we are getting this from the network - 
    # Usually we will get \r\n, but lets make sure...
    $client_str =~ s/[\r\n\s]+$//g;

    if (_ck_pass($client_str, $challenge, $conf->param('passwd'))) {
	_debug('Connection successfully authenticated');
	return 1;
    } else {
	_debug('Authentication failure');
	return 0;
    }
}

sub get_new_conf {
    my ($url, $ua, $res);
    $url = shift;
    _debug("Getting new configuration from $url");

    $ua = LWP::UserAgent->new;
    $res = $ua->request(HTTP::Request->new(GET => $url));
    if ($res->is_success) {
	return $res->content;
    } else {
	warn "Error retreiving file: ",$res->status_line;
	return 0;
    }
}

sub get_old_conf {
    my ($file, $fh, $data);
    $file = shift;
    _debug("Reading old configuration from $file");

    unless ($fh = IO::File->new($file,'r')) {
	warn "Could not read from $file: $!";
	die "Aborting...\n";
    }

    $data = join('', $fh->getlines);

    return $data;
}

sub send_admin_mail {
    my ($from, $to, $server, $body, $date, $smtp, $mail);
    $from = shift;
    $to = shift;
    $server = shift;
    $body = shift;
    _debug("Notifying changes to $to");

    $date = `/bin/date -R`;
    $mail = <<"EOF";
From: IPmanage <$from>
To: $to
Date: $date
Subject: IPmanage: Network configuration changes

$body
EOF

    return ($smtp = Net::SMTP->new($conf->param('smtp_server')) and
	    $smtp->mail($conf->param('send_mail_from')) and
	    $smtp->to($conf->param('send_mail_to')) and
	    $smtp->data($mail));

}

sub write_new_conf {
    my ($file, $fh, $data);
    $file = shift;
    $data = shift;
    _debug("Writing new configuration to $file");

    $fh = IO::File->new($file, 'w') or die $!;
    $fh->print($data);
    $fh->close;

    return 1;
}

sub get_conffile {
    my ($file);

    GetOptions('conf:s' => \$file) or
	die "Error parsing the configuration keys";

    unless (defined $file) {
	$file = '/etc/ipmanage/reconfdhcpd.conf'; # Default value
	warn "Configuration file not specified - Assuming default: $file";
    }

    return $file;
}
sub _debug {
    my ($msg);
    $msg = shift;

    return unless $conf->param('verbose');
    warn "$msg\n";
}

sub _gen_challenge {
    my (@chars, $challenge);
    @chars = ('a'..'z','A'..'Z',0..9);
    $challenge = '';
    while (length($challenge) < 8) {
	$challenge .= $chars[rand @chars];
    }
    return $challenge;
}

sub _ck_pass {
    my ($str, $challenge, $passwd);
    $str = shift;
    $challenge = shift;
    $passwd = shift;

    return 1 if ($str eq md5_hex($challenge, $passwd));

    return 0;
}
