-- $Id: ipmanage.sql 109 2007-04-24 22:19:53Z gwolf $
---------------------
-- ipmanage - ARP/DHCP network management utility
--
-- Copyright 2000-2005 Gunnar Wolf <gwolf@gwolf.org>
--                     Universidad Nacional Autnoma de Mxico
--                     FES Iztacala
--                     Instituto de Investigaciones Econmicas
--
-- Please refer to the LICENSE file in this package for copying details
-- (in short: BSD-like license)
---------------------
-- ipmanage.sql - Database schema for IPManage

DROP TABLE modif_row CASCADE;
DROP TABLE modif CASCADE;
DROP TABLE ip CASCADE;
DROP TABLE network CASCADE;
DROP TABLE config CASCADE;

CREATE TABLE config (
	name text PRIMARY KEY,
	value text NOT NULL
);
REVOKE ALL ON config from PUBLIC;
GRANT ALL ON config to ipmanage;

CREATE TABLE network (
	id serial PRIMARY KEY,
	name text,
	netaddr cidr NOT NULL,
	gateway inet,
	dynamic bool NOT NULL DEFAULT 'f'
);
REVOKE ALL ON network from PUBLIC;
GRANT ALL ON network to ipmanage;
GRANT UPDATE ON network_id_seq to ipmanage;

CREATE TABLE ip (
	ip inet PRIMARY KEY,
	location text,
	responsible text,
	hostname text,
	rev_date timestamp,
	modif_user_id integer,
	details text,
	inventory text,
	mac macaddr UNIQUE,
	blocked bool DEFAULT 't',
	passwd text
);
REVOKE ALL on ip from PUBLIC;
GRANT ALL on ip to ipmanage;

-- Whenever we update an IP, update the revision date
DROP TRIGGER update_rev_date ON ip;
CREATE OR REPLACE FUNCTION update_rev_date () RETURNS trigger AS
'BEGIN
    SELECT INTO NEW.rev_date now()::date;
    RETURN NEW;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER update_rev_date BEFORE INSERT OR UPDATE ON ip
    FOR EACH ROW EXECUTE PROCEDURE update_rev_date();

-- Block any IP address which does not have a corresponding MAC address
-- or does not belong to a dynamic network
DROP TRIGGER block_without_mac ON ip;
CREATE OR REPLACE FUNCTION block_without_mac () RETURNS trigger AS
'DECLARE
    is_dyn bool;
BEGIN
    SELECT INTO is_dyn dynamic FROM network WHERE NEW.ip << network.netaddr;
    IF NOT(is_dyn) AND NEW.mac IS NULL THEN
        NEW.blocked := ''t'';
    END IF;
    RETURN NEW;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER block_without_mac BEFORE INSERT OR UPDATE ON ip
    FOR EACH ROW EXECUTE PROCEDURE block_without_mac();

-- Make sure every registered IP belongs to a registered network
DROP TRIGGER ip_belongs_to_network on ip;
CREATE OR REPLACE FUNCTION ip_belongs_to_network () RETURNS trigger AS
'DECLARE
    parent_net cidr;
BEGIN
    SELECT INTO parent_net netaddr FROM network WHERE new.ip << netaddr;
    IF NOT FOUND THEN
        RAISE EXCEPTION ''IP must be within a defined network'';
    END IF;
    RETURN NEW;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER ip_belongs_to_network BEFORE INSERT OR UPDATE ON ip
    FOR EACH ROW EXECUTE PROCEDURE ip_belongs_to_network();

-- If we get an empty string for any of the values in ip that accept nulls,
-- substitute it for NULL.
-- Note that this will take as empty also strings consisting only of blank
-- spaces.
DROP TRIGGER make_empty_strings_null ON ip;
CREATE OR REPLACE FUNCTION make_empty_strings_null () RETURNS trigger AS
'BEGIN
    IF NEW.location ~ ''^ *$'' THEN NEW.location := NULL; END IF;
    IF NEW.responsible ~ ''^ *$'' THEN NEW.responsible := NULL; END IF;
    IF NEW.details ~ ''^ *$'' THEN NEW.details := NULL; END IF;
    IF NEW.inventory ~ ''^ *$'' THEN NEW.inventory := NULL; END IF;
    IF NEW.hostname ~ ''^ *$'' THEN NEW.hostname := NULL; END IF;
    RETURN NEW;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER make_empty_strings_null BEFORE INSERT OR UPDATE ON ip
    FOR EACH ROW EXECUTE PROCEDURE make_empty_strings_null();

-- If we delete a network, delete all of its IPs as well
DROP TRIGGER delete_ips_with_network ON network;
CREATE OR REPLACE FUNCTION delete_ips_with_network () RETURNS trigger AS
'BEGIN
    DELETE FROM ip WHERE ip << OLD.netaddr;
    RETURN OLD;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER delete_ips_with_network BEFORE DELETE ON network
    FOR EACH ROW EXECUTE PROCEDURE delete_ips_with_network();

-- No two networks are allowed to overlap
-- (Note: As of the CIDR rules, there can not be two overlapping networks 
-- that are not strictly contained one within the other - Fortunately for
-- this script, there can be no intersections. And that is good ;-)
DROP TRIGGER no_overlapping_networks ON network;
CREATE OR REPLACE FUNCTION no_overlapping_networks () RETURNS trigger AS
'DECLARE
    conflict_net cidr;
BEGIN
    SELECT INTO conflict_net netaddr FROM network WHERE 
        netaddr <<= NEW.netaddr AND id != NEW.id;
    IF NOT FOUND THEN
        SELECT INTO conflict_net netaddr FROM network 
            WHERE netaddr >>= NEW.netaddr AND id != NEW.id;
        IF NOT FOUND THEN
            RETURN NEW;
        END IF;
    END IF;
    RAISE EXCEPTION ''Requested network overlaps with %'', conflict_net;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER no_overlapping_networks BEFORE INSERT OR UPDATE ON network
    FOR EACH ROW EXECUTE PROCEDURE no_overlapping_networks();

-- Make sure the gateway is within the subnet, otherwise it would not be
-- reachable. In order to set up an isolated network, set the gateway to
-- NULL.
DROP TRIGGER gateway_in_subnet ON network;
CREATE OR REPLACE FUNCTION gateway_in_subnet () RETURNS trigger AS
'BEGIN
    IF NEW.gateway IS NULL OR NEW.gateway << NEW.netaddr THEN
        RETURN NEW;
    END IF;
    RAISE EXCEPTION ''Gateway % not reachable from network %'', NEW.gateway,
        NEW.netaddr;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER gateway_in_subnet BEFORE INSERT OR UPDATE ON network
    FOR EACH ROW EXECUTE PROCEDURE gateway_in_subnet();

-- Do not allow an existing netaddr to be modified. 
-- If you want to allow for this, please consider:
-- - How to modify every current IP address so it is moved to the new network
-- - What to do if the network shrinks - Should we just discard the remaining
--   IPs?
DROP TRIGGER netaddr_is_immutable ON network;
CREATE OR REPLACE FUNCTION netaddr_is_immutable () RETURNS trigger AS
'BEGIN
    IF NEW.netaddr != OLD.netaddr THEN
        RAISE EXCEPTION ''Network address is immutable - Create a new one.'';
    END IF;
    RETURN NEW;
END;' LANGUAGE 'plpgsql';
CREATE TRIGGER netaddr_is_immutable BEFORE UPDATE ON network
    FOR EACH ROW EXECUTE PROCEDURE netaddr_is_immutable();

-- $Log$
-- Revision 1.7  2005/06/14 23:08:41  gwolf
-- Added hostname handling
--
-- Revision 1.6  2005/05/05 19:40:20  gwolf
-- Implemented a 'modif_user_id' field in ip, killed the modif_log subsystem
-- as it brought too much complexity and was an overkill
--
-- Revision 1.5  2005/05/02 19:17:37  gwolf
-- Changed the structure of modif (still not used, though)
--
-- Revision 1.4  2005/04/19 01:19:44  gwolf
-- - More fine-tuning in network administration
-- - Some more sanity checking in plpgsql
-- - More network-related method (what was it? hmmm... Yup, checking if a
--   network has any static configurations set)
-- - Firsts steps towards gen_dhcpd_conf
--
-- Revision 1.3  2005/04/16 03:31:22  gwolf
-- Functionality for managing networks. This is basically done, yay!
--
-- Revision 1.2  2005/04/06 23:04:35  gwolf
-- Database changes (translation to English, removed some fields, details)
--
-- Revision 1.1  2005/04/02 16:28:20  gwolf
-- Separo dir_ip.sql en user_simple (manejo de usuarios) e ipmanage (sistema
-- propiamente)
--
